@@ -1,3 +1,8 @@
+"""
+testfixtures.comparison
+-----------------------
+"""
+
 from decimal import Decimal
 from difflib import unified_diff
 from functools import partial
@@ -20,6 +25,8 @@ def compare_simple(x, y, context):
         repr_x = repr(x)
         repr_y = repr(y)
         if repr_x == repr_y:
+            if type(x) is not type(y):
+                return compare_with_type(x, y, context)
             x_attrs = _extract_attrs(x)
             y_attrs = _extract_attrs(y)
             diff = _compare_mapping(x_attrs, y_attrs, context, x,
@@ -57,14 +64,21 @@ def _extract_attrs(obj, ignore=None):
         return None
 
     if ignore is not None:
-        if isinstance(ignore, dict):
-            ignore = ignore.get(type(obj), ())
         for attr in ignore:
             attrs.pop(attr, None)
     return attrs
 
 
-def compare_object(x, y, context):
+def _attrs_to_ignore(context, ignore_attributes, obj):
+    ignore = context.get_option('ignore_attributes', ())
+    if isinstance(ignore, dict):
+        ignore = ignore.get(type(obj), ())
+    ignore = set(ignore)
+    ignore.update(ignore_attributes)
+    return ignore
+
+
+def compare_object(x, y, context, ignore_attributes=()):
     """
     Compare the two supplied objects based on their type and attributes.
 
@@ -73,12 +87,17 @@ def compare_object(x, y, context):
        Either a sequence of strings containing attribute names to be ignored
        when comparing or a mapping of type to sequence of strings containing
        attribute names to be ignored when comparing that type.
+
+       This may be specified as either a parameter to this function or in the
+       ``context``. If specified in both, they will both apply with precedence
+       given to whatever is specified is specified as a parameter.
+       If specified as a parameter to this fucntion, it may only be a list of
+       strings.
     """
-    ignore_attributes = context.get_option('ignore_attributes', ())
     if type(x) is not type(y) or isinstance(x, (ClassType, type)):
         return compare_simple(x, y, context)
-    x_attrs = _extract_attrs(x, ignore_attributes)
-    y_attrs = _extract_attrs(y, ignore_attributes)
+    x_attrs = _extract_attrs(x, _attrs_to_ignore(context, ignore_attributes, x))
+    y_attrs = _extract_attrs(y, _attrs_to_ignore(context, ignore_attributes, y))
     if x_attrs is None or y_attrs is None or not (x_attrs and y_attrs):
         return compare_simple(x, y, context)
     if x_attrs != y_attrs:
@@ -573,7 +592,8 @@ def compare(*args, **kw):
     ``actual``.
 
     Any other keyword parameters supplied will be passed to the functions
-    that end up doing the comparison. See the API documentation below
+    that end up doing the comparison. See the
+    :mod:`API documentation below <testfixtures.comparison>`
     for details of these.
 
     :param prefix: If provided, in the event of an :class:`AssertionError`
