@@ -22,7 +22,7 @@ from testfixtures.compat import (
     BytesLiteral, UnicodeLiteral,
     PY2, PY_37_PLUS, ABC
 )
-from testfixtures.comparison import compare_sequence
+from testfixtures.comparison import compare_sequence, compare_object
 from unittest import TestCase
 
 hexaddr = compile('0x[0-9A-Fa-f]+')
@@ -37,36 +37,41 @@ marker = object()
 
 _compare = compare
 
+
+def check_raises(x=marker, y=marker, message=None, regex=None,
+                 compare=compare, **kw):
+    args = []
+    for value in x, y:
+        if value is not marker:
+            args.append(value)
+    for value in 'x', 'y':
+        explicit = 'explicit_{}'.format(value)
+        if explicit in kw:
+            kw[value] = kw[explicit]
+            del kw[explicit]
+    try:
+        compare(*args, **kw)
+    except Exception as e:
+        if not isinstance(e, AssertionError):  # pragma: no cover
+            raise
+        actual = hexsub(e.args[0])
+        if message is not None:
+            # handy for debugging, but can't be relied on for tests!
+            _compare(actual, expected=message, show_whitespace=True)
+            assert actual == message
+        else:
+            if not regex.match(actual):  # pragma: no cover
+                raise AssertionError(
+                    '%r did not match %r' % (actual, regex.pattern)
+                )
+    else:
+        raise AssertionError('No exception raised!')
+
+
 class CompareHelper(object):
 
-    def check_raises(self, x=marker, y=marker, message=None, regex=None,
-                     compare=compare, **kw):
-        args = []
-        for value in x, y:
-            if value is not marker:
-                args.append(value)
-        for value in 'x', 'y':
-            explicit = 'explicit_{}'.format(value)
-            if explicit in kw:
-                kw[value] = kw[explicit]
-                del kw[explicit]
-        try:
-            compare(*args, **kw)
-        except Exception as e:
-            if not isinstance(e, AssertionError):  # pragma: no cover
-                raise
-            actual = hexsub(e.args[0])
-            if message is not None:
-                # handy for debugging, but can't be relied on for tests!
-                _compare(actual, expected=message, show_whitespace=True)
-                assert actual == message
-            else:
-                if not regex.match(actual):  # pragma: no cover
-                    raise AssertionError(
-                        '%r did not match %r' % (actual, regex.pattern)
-                    )
-        else:
-            raise AssertionError('No exception raised!')
+    def check_raises(self, *args, **kw):
+        check_raises(*args, **kw)
 
 
 class TestCompare(CompareHelper, TestCase):
@@ -1806,6 +1811,21 @@ b
             message="Both expected and actual appear as 'Wut', but are not equal!"
         )
 
+    def test_string_with_slotted(self):
+
+        class Slotted(object):
+            __slots__ = ['foo']
+            def __init__(self, foo):
+                self.foo = foo
+            def __repr__(self):
+                return repr(self.foo)
+
+        self.check_raises(
+            'foo',
+            Slotted('foo'),
+            "'foo' (%s) != 'foo' (%s)" % (repr(str), repr(Slotted))
+        )
+
     def test_not_recursive(self):
         self.check_raises(
             {1: 'foo', 2: 'foo'},
@@ -1861,6 +1881,34 @@ class TestIgnore(CompareHelper):
         )
 
 
+class TestCompareObject(object):
+
+    class Thing(object):
+        def __init__(self, **kw):
+            for k, v in kw.items():
+                setattr(self, k, v)
+
+    def test_ignore(self):
+        def compare_thing(x, y, context):
+            return compare_object(x, y, context, ignore_attributes=['y'])
+        compare(self.Thing(x=1, y=2), self.Thing(x=1, y=3),
+                comparers={self.Thing: compare_thing})
+
+    def test_ignore_dict_context_list_param(self):
+        def compare_thing(x, y, context):
+            return compare_object(x, y, context, ignore_attributes=['y'])
+        compare(self.Thing(x=1, y=2, z=3), self.Thing(x=1, y=4, z=5),
+                comparers={self.Thing: compare_thing},
+                ignore_attributes={self.Thing: ['z']})
+
+    def test_ignore_list_context_list_param(self):
+        def compare_thing(x, y, context):
+            return compare_object(x, y, context, ignore_attributes=['y'])
+        compare(self.Thing(x=1, y=2, z=3), self.Thing(x=1, y=4, z=5),
+                comparers={self.Thing: compare_thing},
+                ignore_attributes=['z'])
+
+
 class BaseClass(ABC):
     pass
 
